# BDM commands

#	Copyright (C) 2003 Embedded CL S.A.
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from bdmpod import *
from bdmutil import *

DEFAULT_RETRIES = 2 # for NOT_READY results

# transmit

# packet fields
REG_BITS = (2,0)
AD_BIT = 3
SIZE_BITS = (7,6)
RW_BIT = 8
OP_BITS = (15,10)

# AnD
AR = 1<<3
DR = 0<<3

# SIZE
BYTE = B = 0x00
WORD = W = 0x40
LONG = L = 0x80

# RnW
RD = 1<<8
WR = 0<<8

# OPs
REG = 0x2000 # register
MEM = 0x1800 # memory
BLK = 0x1c00 # memory block
CREG = 0x2800 # control register
DREG = 0x2c00 # debug register

# explicit ops (commands)
RAREG = 0x2180 | 1<<3 # REG|AR|RD|LWORD
RDREG = 0x2180 | 0<<3 # REG|DR|RD|LWORD
WAREG = 0x2080 | 1<<3 # REG|AR|WR|LWORD
WDREG = 0x2080 | 0<<3 # REG|DR|WR|LWORD
READ = 0x1900 # B, W, L
WRITE = 0x1800 # B, W, L
DUMP = 0x1d00 # B, W, L
FILL = 0x1c00 # B, W, L
GO = 0x0c00
NOP = 0x0000
SYNC_PC = 0x0001
RCREG = 0x2980
WCREG = 0x2880
RDMREG = 0x2d80 # DRc[4:0]
WDMREG = 0x2c80 # DRc[4:0]

RDB = -(READ|BYTE)
RDW = -(READ|WORD)
RDL = -(READ|LONG)

WRB = -(WRITE|BYTE)
WRW = -(WRITE|WORD)
WRL = -(WRITE|LONG)

DPB = -(DUMP|BYTE)
DPW = -(DUMP|WORD)
DPL = -(DUMP|LONG)

FLB = -(FILL|BYTE)
FLW = -(FILL|WORD)
FLL = -(FILL|LONG)


class bdmcmd:
	# Description dictionary
	# key: (#arguments, name, description)

	__desc = {
		RAREG: (1, 'RAREG', 'Read address register'),
		RDREG: (1, 'RDREG', 'Read data register'),
		WAREG: (2, 'WAREG','Write address register'),
		WDREG: (2, 'WDREG','Write data register'),
		READ: (2, 'READ','Read memory'),
		WRITE: (3, 'WRITE','Write memory'),
		DUMP: (3, 'DUMP','Read memory block'),
		FILL: (4, 'FILL','Write memory block'),
		GO: (0, 'GO','Resume execution at current PC'),
		NOP: (0, 'NOP','No operation'),
		SYNC_PC: (0, 'SYNC_PC','Capture PC and display it on PSTDDATA'),
		RCREG: (1, 'RCREG','Read system control register'),
		WCREG: (2, 'WCREG','Write system control register'),
		RDMREG: (1, 'RDMREG','Read debug moduel register'),
		WDMREG: (2, 'WDMREG','Write debug moduel register')
	}

	def __init__(self, bp, rt=DEFAULT_RETRIES):
		self.pod = bp
		self.retries = rt

	def __call__(self, *v):
		m = self.__opcls[v[0]]
		return m(self, *v)


	def decode_op(self, *v):
		op = get_bits(-v[0], OP_BITS)<<OP_BITS[1]
		rw = get_bit(-v[0], RW_BIT)<<RW_BIT
		sz = get_bits(-v[0], SIZE_BITS)<<SIZE_BITS[1]
		return self(*(op|rw, sz) + v[1:])

	# GO NOP SYNC_PC
	def single_op(self, *v):
		p = self.pod
		p(v[0])
		return None

	# RAREG RDREG
	def read_reg(self, *v):
		p = self.pod
		p(v[0]|v[1])
		r = p(NOP)<<16
		r |= p(NOP)
		return r

	# WAREG WDREG
	def write_reg(self, *v):
		p = self.pod
		p(v[0]|v[1])
		p((v[2]>>16) & 0xffff)
		p(v[2] & 0xffff)
		return None

	# RCREG RDMREG
	def read_creg(self, *v):
		p = self.pod
		p(v[0])
		p((v[1]>>16) & 0xffff)
		p(v[1] & 0xffff)
		r = p(NOP)<<16
		return r | p(NOP)

	# WCREG WDMREG
	def write_creg(self, *v):
		p = self.pod
		p(v[0])
		p((v[1]>>16) & 0xffff)
		p(v[1] & 0xffff)
		p((v[2]>>16) & 0xffff)
		p(v[2] & 0xffff)
		rt = self.retries+1
		while rt:
			r = p(NOP)
			if r != NOT_READY: return None # ok
			rt -= 1
		raise BdmError(r)

	# READ|BWL (READ, BWL, adr)
	def read_mem(self, *v):
		p = self.pod
		p(v[0]|v[1])
		p((v[2]>>16) & 0xffff)
		p(v[2] & 0xffff)
		rt = self.retries+1
		while 1:
			r = p(NOP)
			if r != NOT_READY: break # ok
			rt -= 1
			if not rt: raise BdmError(r)
		if v[1] == BYTE: return r & 0xff
		elif v[1] == WORD: return r
		else: # LONG
			r <<= 16
			return r | p(NOP) # retry?

	# WRITE|BWL (WRITE, BWL, adr, val)
	def write_mem(self, *v):
		p = self.pod
		p(v[0]|v[1])
		p((v[2]>>16) & 0xffff)
		p(v[2] & 0xffff)
		if v[1] == BYTE: p(v[3] & 0xff)
		elif v[1] == WORD: p(v[3] & 0xffff)
		else: # LONG
			p((v[3]>>16) & 0xffff)
			p(v[3] & 0xffff)
		return None

	# DUMP|BWL (DUMP, BWL)
	def _dump_mem(self, *v):
		p = self.pod
		p(v[0]|v[1])
		r = p(NOP)
		if v[1] == BYTE: return r & 0xff
		elif v[1] == WORD: return r
		else: # LONG
			r <<= 16
			return r | p(NOP)

	# DUMP|BWL (DUMP, BWL, adr, cnt)
	def dump_mem(self, *v):
		if v[1] == LONG: return self.dump_mem_l(*v)
		else: return self.dump_mem_bw(*v)

	def dump_mem_bw(self, *v):
		from array import array
		if v[1] == BYTE: c = 'B'; m = 0xff
		else: c = 'H'; m = 0xffff
		if v[3] == 0: return array(c)
		r = array(c, [0]*v[3])
		# r.append(self.read_mem(READ, v[1], v[2]))
		r[0] = self.read_mem(READ, v[1], v[2])
		if v[3] == 1: return r
		i, cnt, p = 1, v[3]-1, self.pod
		p(DUMP|v[1])
		while i<cnt:
			t = p(DUMP|v[1])
			# while t == NOT_READY: t = p(NOP)
			rt = self.retries+1
			while t == NOT_READY:
				rt -= 1
				if not rt: raise BdmError(r)
				t = p(NOP)
			# r.append(t & m)
			r[i] = t & m
			i += 1
		t = p(NOP)
		# while t == NOT_READY: t = p(NOP)
		rt = self.retries+1
		while t == NOT_READY:
			rt -= 1
			if not rt: raise BdmError(r)
			t = p(NOP)
		# r.append(t & m)
		r[i] = t & m
		return r

	def dump_mem_l(self, *v):
		from array import array
		if v[3] == 0: return array('l')
		r = array('l', [0]*v[3])
		# r.append(self.read_mem(READ, v[1], v[2]))
		r[0] = self.read_mem(READ, v[1], v[2])
		if v[3] == 1: return r
		i, cnt, p = 1, v[3]-1, self.pod
		p(DUMP|v[1])
		while i<cnt:
			t = p(NOP)
			# while t == NOT_READY: t = p(NOP)
			rt = self.retries+1
			while t == NOT_READY:
				rt -= 1
				if not rt: raise BdmError(r)
				t = p(NOP)
			u = p(DUMP|v[1])
			# r.append((t<<16)|u)
			r[i] = (t<<16)|u
			i += 1
		t = p(NOP)
		# while t == NOT_READY: t = p(NOP)
		rt = self.retries+1
		while t == NOT_READY:
			rt -= 1
			if not rt: raise BdmError(r)
			t = p(NOP)
		u = p(NOP)
		# r.append((t<<16)|u)
		r[i] = (t<<16)|u
		return r

	# FILL|BWL (FILL, BWL, val)
	def _fill_mem(self, *v):
		p = self.pod
		p(v[0]|v[1])
		if v[1] == BYTE: p(v[2] & 0xff)
		elif v[1] == WORD: p(v[2] & 0xffff)
		else: # LONG
			p((v[2]>>16) & 0xffff)
			p(v[2] & 0xffff)
		return None

	def fill_mem(self, *v):
		if v[1] == LONG: return self.fill_mem_l(*v)
		else: return self.fill_mem_bw(*v)

	# FILL|BW (FILL, BW, adr, array)
	def fill_mem_bw(self, *v):
		if not len(v[3]): return None
		if v[1] == BYTE: c = 'B'; m = 0xff
		else: c = 'H'; m = 0xffff
		self.write_mem(WRITE, v[1], v[2], v[3][0])
		if len(v[3]) == 1: return None
		p = self.pod
		cnt = len(v[3]); i = 1
		while i < cnt:
			p(FILL|v[1])
			p(v[3][i] & m)
#			while t == NOT_READY: t = p(DUMP|v[1])
			i += 1
#		t = p(NOP)
#		while t == NOT_READY: t = p(NOP)
		return None

	# FILL|BW (FILL, BW, adr, array)
	def fill_mem_l(self, *v):
		if not len(v[3]): return None
		self.write_mem(WRITE, v[1], v[2], v[3][0])
		if len(v[3]) == 1: return None
		p = self.pod
		cnt = len(v[3]); i = 1
		while i < cnt:
			p(FILL|v[1])
			p(v[3][i]>>16 & 0xffff)
			p(v[3][i] & 0xffff)
			i += 1
		return None

	# dispatch method table
	__opcls = {
		RAREG: read_reg,
		RDREG: read_reg,
		WAREG: write_reg,
		WDREG: write_reg,
		READ: read_mem,
		WRITE: write_mem,
		DUMP: dump_mem, # self.read_blk,
		FILL: fill_mem,
		GO: single_op,
		NOP: single_op,
		SYNC_PC: single_op,
		RCREG: read_creg,
		WCREG: write_creg,
		RDMREG: read_reg,
		WDMREG: write_reg,
		RDB: decode_op,
		RDW: decode_op,
		RDL: decode_op,
		WRB: decode_op,
		WRW: decode_op,
		WRL: decode_op,
		DPB: decode_op,
		DPW: decode_op,
		DPL: decode_op,
		FLB: decode_op,
		FLW: decode_op,
		FLL: decode_op
	}



#	# DUMP block (DUMP, BWL, adr, cnt)
#	def read_blk(self, *v):
#		from array import array
#		if v[1] == BYTE: c = 'B'
#		elif v[1] == WORD: c = 'H'
#		else: c = 'l'
#		r = array(c)
#		r.append(self.read_mem(READ, v[1], v[2]))
#		cnt = v[3]-1
#		while cnt:
#			r.append(self.dump_mem(DUMP, v[1]))
#			cnt -= 1
#		return r

#	def rareg(self, v):
#		r = self.pod([RAREG | v & 0x7, NOP, NOP])
#		return r[1]<<16 | r[2];


#	def __call__(self, *v):
##		if v[0] not in __desc.keys(): raise ValueError('undefined command')
##		if len(v)-1 != __desc[v[0]][0]: raise TypeError('bad number of arguments')
#
#		if v[0]==RAREG or v[0]==RDREG:
#			r = self.pod([v[0] | v[1] & 0x7, NOP, NOP])
#			return r[1]<<16 | r[2]
#
#		if v[0]==RCREG:
#			r = self.pod([v[0], NOP, NOP])
#			return r[1]<<16 | r[2]

#	def build_operand(self, *v):
##		op = get_bits(v[0], OP_BITS)<<OP_BITS[1]
##		rw = get_bit(v[0], RW_BIT)
##		sz = get_bits(v[0], SIZE_BITS)
##		ad = get_bit(v[0], AD_BIT)
#		m = self.__opcls[v[0]]
#		if m == self.read_reg: return v[0]|v[1] # (REGOP, REG)
#		elif m = self.write_reg: return v[0]
#		else: return v[0]

#	def fetch_command(self, c, i):
#		if type(c[i]) is tuple: return c[i][0] # op with args
#		else: return c[i] # single op

	# execute a list of commands [(cmd, data), cmd, ... ]
	# map(lambda x: <bdmcmd instance>(*x), cmd_list)
#	def run(self, c):
#		map(self.pod.transfer, c)
#		for i in range(0, len(c)):
#			rt = self.retries
#			while rt >= 0:
#				try:
#					self.run_command(c, i)
#					break
#				except BdmError, e:
#					if e[0] == NOT_READY:
#						rt -= 1
#						if rt < 0: raise e # dispatch
#					elif self.mode == REPORT_MODE:
#						c[i] = e # report it
#						break
#					else: raise e # abort


#	def run_command(self, c, i):
#		cm = fetch_command(c, i) # this command
##		op = get_bits(cm, OP_BITS)<<OP_BITS[1]
##		rw = get_bit(cm, RW_BIT)
#		if i == len(c)-1: cn = NOP # next command
#		else: cn = fetch_command(c, i+1)
#		m = self.__opcls[cm]
#
#		c[i] = m(*(cm, cn))
#		return
